###############################################################################
###############################################################################
#####                            Ventana.tcl
###############################################################################
##### Creates the window which will provide some information to the user 
##### while a file is being downloaded.
###############################################################################
##### Copyright 1999-2007 Andrs Garca Garca - fandom@telefonica.net
##### Distributed under the terms of the GPL v2
###############################################################################

namespace eval Ventana {

source [file join $dirGetleft(scripts) Rizo.tcl]

set family [font configure TkTextFont -family]
set size   [font configure TkTextFont -size] 

catch {font create labelFont -family $family -size $size -weight bold}

###############################################################################
# DefineButtonImages
#    Reads the images needed to create the 'downloading' window.
###############################################################################
proc DefineButtonImages {} {
    global   dirGetleft getleftState
    variable images

    set ext $getleftState(imgExt)

    set images(url)    [image create photo                             \
            -file [file join "$dirGetleft(icons)" html.$ext]]
    set images(folder) [image create photo                             \
            -file [file join "$dirGetleft(icons)" folder.$ext]]
    set images(files)  [image create photo                             \
            -file [file join "$dirGetleft(icons)" files.$ext]]
    set images(clock)  [image create photo                             \
            -file [file join "$dirGetleft(icons)" clock.$ext]]

    set images(pause)  [image create photo                             \
            -file  [file join "$dirGetleft(icons)" pauses.$ext]]
    set images(resume) [image create photo                             \
            -file  [file join "$dirGetleft(icons)" resumes.$ext]]
    set images(stop)   [image create photo                             \
            -file  [file join "$dirGetleft(icons)" stops.$ext]]
    set images(conf)   [image create photo                             \
            -file  [file join "$dirGetleft(icons)" confs.$ext]]

    return
}

###############################################################################
# ChangeProgressBar
#     During a download we need a determinate progress bar, but while 
#     connecting or processing a file an indeterminate one is better.
#
# Parameter:
#     action: 'start' or 'stop'. 
###############################################################################
proc  ChangeProgressBar {action} {
    variable window

    if {$action=="start"} {
        $window(bar) configure -mode indeterminate
        $window(bar) start
    } else {
        $window(bar) stop
        set Ventana::Rizo::curlReport(percentage) 0
        $window(bar) configure -mode determinate
    }

    return
}

###############################################################################
# ShowResumeButton
#    When the user pauses the download, the 'Pause' menu button is 
#    subtituted by the 'Resume' button
###############################################################################
proc ShowResumeButton {} {
    variable window
    variable menus
    global   getleftState

    $window(pause) configure -state disabled
    place $window(resume) -in $window(pause)
    $menus(stop) entryconfigure 3 -state disabled

    return
}

###############################################################################
# HideResumeButton
#    And when the download is resumed it is hidden again.
###############################################################################
proc HideResumeButton {} {
    variable window
    variable menus

    place forget $window(resume)
    $window(pause) configure -state normal

    $menus(stop) entryconfigure 3 -state normal

    return
}

###############################################################################
# PauseDownloading
#    Pauses the downloading according to the menu entries.
#
# Parameter
#    which: either 'file' or 'page'
###############################################################################
proc Pause {which} {
    global getleftOptions labelTitles labelMessages
    variable window

    set windowState [wm state $window(top)]
    if {$windowState=="iconic"} {
        wm deiconify $window(top)
    }

    ShowResumeButton

    if {$which=="file"} {
        tkwait variable getleftOptions(pauseFile)
    } else {
        tkwait variable getleftOptions(pausePage)
    }

    return
}

###############################################################################
# CancelDownloading
#    This procedure is invoked from the downloading window when you click on
#    the 'cancel' button or you want to close the window
#
# Parameter:
#    parent: The window over which the asking dialog will appear.
#    skip: if skip is 'skip' the current file will be skipped and the next one
#    will begin to be downloaded. If there is no parameter, the downloading
#    will be stopped.
#
# Returns
#    '0' if we are not going to stop.
#    '1' if we are.
###############################################################################
proc CancelDownloading {parent {skip {} }} {
    global labelTitles labelMessages
    global getleftState getleftOptions
    global ::Ventana::Rizo::curlReport
    variable waitingPid
    variable window   

    if {$getleftState(delayedDown)==0} {
        set what [Dialogos::Dialogo $parent.error -type yesno -icon question \
                -title $labelTitles(confirm)                                 \
                -message $labelMessages(stopDownload)]
        if {$what=="no"} {
            return 0
        }
        if {[info exists window(bar)]} {
            $window(bar) configure -value 0
        }
        set Ventana::Rizo::curlReport(left) ""
    }
    catch {after cancel $waitingPid}
    catch {Rizo::Lector stopNow}

    set getleftOptions(cancelDown)     1
    set Ventana::Rizo::curlReport(end) 1
    set getleftState(autoDown)         0

    if {($skip=="")||($getleftState(filesChosen)==0)} {
        set getleftOptions(stopFile)  1
        set getleftState(downloading) 0
        catch {wm withdraw $window(top)}
    }

    incr curlReport(nextFile)

    if {$getleftState(waiting)==1} {
        set getleftState(waiting) 0
    }
    return 1
}

###############################################################################
# TrimString
#    Trims the given string so that if it is longer than the available space
#    the middle of it gets substituted with '...', and puts the resulting
#    string in the given label.
#
# Parmetros
#    cadena: string to trim.
#    labelPath: the label in which the string is going to be shown
###############################################################################
proc TrimString {cadena labelPath} {

    set font     labelFont
    set varName  [$labelPath cget -textvariable]

    upvar #0 $varName textVariable

    set textVariable $cadena
    set labelLength  [expr {[winfo width $labelPath]-5}]
    set strLength    [font measure $font -displayof $labelPath $cadena]

    if {$labelLength<=$strLength} {
        set strChars [string length $cadena]
        set middle   [expr {round($strChars/2)-2}]
        set primer   [string range $cadena 0 $middle]
        set segun    [string range $cadena [expr {$middle+1}] end]
        set textVariable ""
        append textVariable $primer "..." $segun
        while {[font measure $font -displayof $labelPath $textVariable]>$labelLength} {
            regsub {(.)(\.\.\.)(.)} $textVariable {...} textVariable
        }
        BalloonHelp::set_balloon $labelPath $cadena
    } else {
        BalloonHelp::delete_balloon $labelPath
    }
    return
}

###############################################################################
# FileStrings
#    Controls the strings that will be put in the downloading window, to
#    show the name and directory of the file being processed or downloaded
#
# Parameter:
#    url to be downloaded or preprocessed
###############################################################################
proc FileStrings {url} {
    global directories labelMessages
    variable window

    set fileName  [::Commands::UrlToFile $url $directories(base)]
    set parsedUrl [HtmlParser::ParseUrl $url]
    set domain    [string tolower [lindex $parsedUrl 1]]
    set domainTmp [string map {: ""} $domain]
    if {[regexp -nocase "(?:$directories(base)/$domainTmp/)(.*)" \
            $fileName nada tmp]==0} {
        set tmp ""
    }

    if {![regexp {(.*)(?:/)(.*)} $tmp nada tmpDir tmpFile]} {
        set tmpDir ""
        set tmpFile $tmp
    }

    TrimString $domain  $window(url)
    TrimString $tmpDir  $window(dir)
    TrimString $tmpFile $window(file)

    update

    return
}

###############################################################################
# DownloadWindow
#    Creates the window, in which some info about the downloading will be
#    displayed.
###############################################################################
proc DownloadWindow {} {
    global labelButtons labelDialogs labelMenus dirGetleft
    global getleftOptions getleftState
    variable window
    variable menus
    variable images

    DefineButtonImages

    set win [toplevel .bajando]
    wm resizable $win 0 0
    wm maxsize $win 299 200

    if {$getleftState(os)=="mac"} {
        set labelWidth  27
    } else {
        set labelWidth 35
    }

    set marco     [ttk::frame $win.marco -class ExternalFrame]
    set marco_in  [ttk::frame $marco.marco_interno]
    set marco_url [ttk::frame $marco_in.url]
    set marco_dir [ttk::frame $marco_in.dir]
    set marco_fil [ttk::frame $marco_in.fil]
    set marco_que [ttk::frame $marco_in.que]
    set marco_bar [ttk::frame $marco_in.bar]
    set exButtons [ttk::frame $win.exButtons -class ExternalFrame]
    set botones   [ttk::frame $exButtons.botones]

    set etq_url   [ttk::label $marco_url.etq_url  -image $images(url)]
    set etq_dir   [ttk::label $marco_dir.etq_dir  -image $images(folder)]
    set etq_file  [ttk::label $marco_fil.etq_file -image $images(files)]
    set etq_que   [ttk::label $marco_que.etq_que  -image $images(clock)]

    BalloonHelp::set_balloon $etq_url  $labelDialogs(url)
    BalloonHelp::set_balloon $etq_dir  $labelDialogs(dir)
    BalloonHelp::set_balloon $etq_file $labelDialogs(file)
    BalloonHelp::set_balloon $etq_que  $labelDialogs(left)

    set etq_url2  [ttk::label $marco_url.etq_url2  -width $labelWidth -anchor w   \
            -relief sunken -textvariable Ventana::window(labelUrl) -font labelFont]
    set etq_dir2  [ttk::label $marco_dir.etq_dir2  -width $labelWidth -anchor w   \
            -textvariable Ventana::window(labelDir)       -font labelFont]
    set etq_file2 [ttk::label $marco_fil.etq_file2 -width $labelWidth -anchor w   \
            -textvariable Ventana::window(labelFile)      -font labelFont]
    set etq_que2  [ttk::label $marco_que.etq_que2  -width $labelWidth -anchor w   \
            -textvariable Ventana::Rizo::curlReport(left) -font labelFont]

    set progressBar [ttk::progressbar $marco_bar.progress -orient horizontal \
            -length 275 -mode determinate                                    \
            -variable Ventana::Rizo::curlReport(percentage)]

    set window(top)  $win
    set window(url)  $etq_url2
    set window(dir)  $etq_dir2
    set window(file) $etq_file2
    set window(left) $etq_que2
    set window(bar)  $progressBar

    set config   [ttk::button     $botones.config -image $images(conf)          \
            -command "mainWin::UrlListChangeOptions $window(top)" -style Toolbutton]
    set pause    [ttk::menubutton $botones.pause -menu $botones.pause.menu      \
            -image $images(pause) -style Toolbutton]
    set stop     [ttk::menubutton $botones.stop  -menu $botones.stop.menu       \
            -image $images(stop) -style Toolbutton]
    set resume   [ttk::button     $botones.resume -image $images(resume)        \
            -command "::Ventana::ResumeDownloading" -style Toolbutton]

    ::BalloonHelp::set_balloon $config $labelDialogs(options)
    ::BalloonHelp::set_balloon $pause  $labelDialogs(pause)
    ::BalloonHelp::set_balloon $stop   $labelDialogs(stop)
    ::BalloonHelp::set_balloon $resume $labelDialogs(start)

    set menus(pause) [menu $pause.menu -tearoff 0 -fg black -bg $getleftOptions(menus) \
            -postcommand "
                BalloonHelp::kill_balloon
    "]
    $menus(pause) add command -label $labelMenus(stopNow) -command {
            Ventana::PauseDownloading now
    }
    $menus(pause) add check -variable getleftOptions(pausePage)              \
            -onvalue 1 -offvalue 0 -label $labelMenus(stopPage)
    $menus(pause) add check -variable getleftOptions(pauseFile)              \
            -onvalue 1 -offvalue 0 -label $labelMenus(stopFile)

    set menus(stop) [menu $stop.menu -tearoff 0 -fg black -bg $getleftOptions(menus) \
            -postcommand "
                    BalloonHelp::kill_balloon
    "]
    $menus(stop) add command -label $labelMenus(stopNow) -command {
        if {$getleftState(delayedDown)==1} Delay::DelayedCancelDelay
        ::Ventana::CancelDownloading $Ventana::window(top)
    }
    $menus(stop) add check -variable getleftOptions(stopPage)                \
            -onvalue 1 -offvalue 0 -label $labelMenus(stopPage)              \
            -command {
                set getleftState(autoDown) 0
                if {$getleftState(delayedDown)==1} Delay::DelayedCancelDelay
            }
    $menus(stop) add check -variable getleftOptions(stopFile)                \
            -onvalue 1 -offvalue 0 -label $labelMenus(stopFile)              \
            -command {
                set getleftState(autoDown) 0
                if {$getleftState(delayedDown)==1} Delay::DelayedCancelDelay
            }
    $menus(stop) add command -label $labelButtons(skip)                      \
            -command "::Ventana::CancelDownloading $Ventana::window(top) skip"

    set window(cancel) $stop
    set window(pause)  $pause
    set window(resume) $resume

    pack $marco     -ipady 5
    pack $exButtons -fill x
    pack $botones   -fill x -side right -padx 7
    pack $marco_in  -padx 10 -side bottom
    pack $marco_url $marco_in $marco_dir $marco_fil $marco_que $marco_bar \
            -fill x
    pack $etq_url   $etq_url2  -side left -pady 2 -padx 2
    pack $etq_dir   $etq_dir2  -side left -pady 2 -padx 2
    pack $etq_file  $etq_file2 -side left -pady 2 -padx 2
    pack $etq_que   $etq_que2  -side left -pady 2 -padx 2
    pack $progressBar
    pack $stop $pause $config -side right -padx 3 -pady 5

    wm protocol $win WM_DELETE_WINDOW "::Ventana::CancelDownloading $win"

    return
}

###############################################################################
# ShowWindow
#   If needed, maps the downloading window, and sets its title bar
#
# Parameter
#   which: either 'head', 'down', 'process' or 'wait' depending on the action 
#          taking place.
###############################################################################
proc ShowWindow {which} {
    global   labelTitles labelButtons getleftState
    variable window

    if {$::getleftState(downloading)==0} {
        if {$::DEBUG==1} {
            Dialogos::Dialogo .error -title Error -icon error -type ok       \
                    -message "Pringamos en ShowWindow: $which"
        }
        return
    }

    if {![winfo exists .bajando]} {
        DownloadWindow
    } else {
        set state [wm state $window(top)]
        if {$state=="withdrawn"} {
            wm deiconify $window(top)
        }
    }

    switch -exact $which {
        head {
            wm title $window(top) $labelTitles(connect)
            ChangeProgressBar start
        }
        down {
            wm title $window(top) $labelTitles(download)
            ChangeProgressBar stop
        }
        process {
            wm title $window(top) $labelTitles(process)
            ChangeProgressBar start
        }
        wait {
            StartWaiting 10
        }
    }

    update

    return
}

###############################################################################
# FileDownloading
#   Creates a nice window to show info about how the downloading goes
#
# Parameters:
#    fichero: file in which the link will be saved.
#    enlace:  file url.
#    madre:   referer page for the link
###############################################################################
proc FileDownloading {fichero enlace madre} {
    global   getleftState directories
    global   labelTitles
    global   Ventana::Rizo::meta
    global   Ventana::Rizo::curlReport
    variable window
    variable file
    variable link
    variable mother
    variable action

    if {$getleftState(downloading)==0} {
        return
    }

    set file   $fichero
    set action body

    ShowWindow down
    FileStrings $enlace

    set curlReport(end)   0
    set curlReport(long)  1
    set curlReport(pause) 0

    if {([file exists $fichero])&&($meta(versionServer)>=1.1)} {
        set size [file size $fichero]
        if {($size>$meta(totalBytes))&&($meta(totalBytes)!=0)} {
            file delete $fichero
        } elseif {($size==$meta(totalBytes))&&($meta(totalBytes)!=0)} {
            after 1000 "set ::Ventana::Rizo::curlReport(nextFile) 1"
            return
        }
        Ventana::Rizo::DataRequest $fichero $enlace $madre 1
    } else {
        Ventana::Rizo::DataRequest $fichero $enlace $madre
    }
    CheckDownload

    return
}

###############################################################################
# HeadDownloading
#    Gets the headers of the url from the server.
#
# Parameters:
#    enlace: url to download
#    madre:  referer page of the link
###############################################################################
proc HeadDownloading {enlace madre} {
    global siteUrl getleftState
    variable link
    variable mother
    variable action
    variable window

    set link   $enlace
    set mother $madre
    set action head

    ShowWindow head
    FileStrings $enlace

    HideResumeButton

    if {[regexp {^ftp://} $enlace]} {
        set ::Ventana::Rizo::meta(content)       "nada"
        set ::Ventana::Rizo::meta(totalBytes)    0
        set ::Ventana::Rizo::meta(versionServer) 1.1
        set ::Ventana::Rizo::meta(relocate)      ""
        set ::Ventana::Rizo::curlError           0
        set ::Ventana::Rizo::errorMessage        ""
        return
    }

    set Ventana::Rizo::curlReport(end) 0
    Ventana::Rizo::HeadRequest $enlace $madre
    tkwait variable Ventana::Rizo::curlReport(end)

    return
}

###############################################################################
# PauseDownloading
#    Takes control when you click on the 'Pause' button.
#
# Parameter:
#    when: 'now' when we click on the 'Now' menu entry, usuful to pause
#    the downloading while a file is being process for links.
###############################################################################
proc PauseDownloading {{when ""}} {
    global labelButtons getleftOptions
    variable window

    if {$when=="now"} {
        set getleftOptions(pauseNow) 1
        set Ventana::Rizo::curlReport(pause) 1
    }
    set Ventana::Rizo::curlReport(pause) 1
    catch {Rizo::Lector stopNow}
    ShowResumeButton

    return
}

###############################################################################
# ResumeDownloading
#    Takes control when you click on the 'Resume' button
###############################################################################
proc ResumeDownloading {} {
    global labelButtons getleftOptions
    variable window
    variable file
    variable link
    variable mother
    variable action

    HideResumeButton

    if {($getleftOptions(pauseFile)==1)||($getleftOptions(pausePage)==1)} {
        set getleftOptions(pauseFile) 0
        set getleftOptions(pausePage) 0
        return
    }
    set getleftOptions(pauseNow)              0
    set Ventana::Rizo::curlReport(pause)      0
    set Ventana::Rizo::curlReport(percentage) 0

    if {$action=="body"} {
        Ventana::Rizo::DataRequest "$file" "$link" "$mother" 1
        Ventana::CheckDownload
    } else {
        HeadDownloading "$link" "$mother"
    }
    return
}

###############################################################################
# StartWaiting
#    This procedure is invoked when a timeout has ocurred, to wait before
#    trying again.
#
# Parameter
#    seconds: The seconds yet to wait
###############################################################################
proc StartWaiting {seconds} {
    global   getleftState labelTitles
    variable window
    variable waitingPid

    if {$seconds==0} {
        InterruptWaiting
        return
    }

    wm title $window(top) "$labelTitles(waiting) $seconds"

    ShowResumeButton
    $window(resume) configure -command ::Ventana::InterruptWaiting

    set waitingPid [after 1000 ::Ventana::StartWaiting [expr $seconds -1]]

    return
}

###############################################################################
# InterruptWaiting
#    While waiting to retry after a timeout, if you click on resume
#    this procedure will be invoked.
###############################################################################
proc InterruptWaiting {} {
    global   getleftState
    variable window
    variable waitingPid

    after cancel $waitingPid

    HideResumeButton
    $window(resume) configure -command ::Ventana::ResumeDownloading

    set getleftState(waiting) 0

    return
}

###############################################################################
# CheckDownload
#    The procedure get invoked from time to time in order to check whether
#    the transfer has ended, with success or not.
###############################################################################
proc CheckDownload {} {
    global Ventana::Rizo::curlReport
    global Ventana::Rizo::curlError
    global Ventana::Rizo::meta
    global labelDialogs downOptions getleftOptions
    variable file
    variable link
    variable mother
    variable window

    if {[string match $curlReport(percentage) ""]} return
    if {$getleftOptions(pauseNow)==1} {
        return
    }
    if {($curlReport(end)==0) && ($curlReport(stop)!=1) \
            && ($curlReport(pause)==0)} {
        set afterId [after 750 Ventana::CheckDownload]
    } else {
        set curlReport(left) ""
        if {($::DEBUG==1)&&($curlError!=0)} {
            puts "Error $curlError en $file"
        }
        if {($curlError==18)||($curlError==28)} {
            getLog::FileLogEnter $labelDialogs(fileIncom) \
                    "$link" "$mother"
            if {($downOptions(resume)==1)&&($curlReport(stop)!=1)} {
                $window(bar) stop
                set ::Ventana::Rizo::curlReport(percentage) 0
                if {$curlError==18} {
                    FileDownloading "$file" "$link" "$mother"
                } else {
                    incr curlReport(nextFile)
                }
                return
            }
        }
        if {$curlError==0} {
            $window(bar) configure -value 100
            update
        }
        after 75
        $window(bar)  configure -value 0
        set ::Ventana::Rizo::curlReport(percentage) 0
        $window(dir)  configure -text ""
        $window(file) configure -text ""
        incr curlReport(nextFile)
    }
    return
}
}
